%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fit SQ CPMG to Carver-Richards equation
% Residue by residue fitting 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INSTRUCTION FOR INPUT FILE (save them in the directory "inputs")
% 2 input files are required for the data at high static field (file name: HighField.in) and low static field (file name: LowField.in), respectively 
%
%The first column of each input file is the list of cpmg fields (note the
%first number is not read - enter 9999)
%
%columns 2 to N+1 are the R2 values at each cpmg filed for the N analyzed
%NMR peaks - if you miss one or more values, set them to a number larger
%than 300 (they won't be used for fitting - the first entry of each column 
%is the peak index (not an R2 value) - Peaks need to be listed in the same
%order in the HighFiled.in and LowField.in files
%
%columns N+2 to 2N+1 are the errors associated with the R2 values - the first entry of each column 
%is the peak index
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SETTING UP THE FITTING SCRIPT
%set NumRes to the number of NMR peaks in the fit
%set NumFields to the number of cpmg fields
%set FULL_CT to the relaxation delay period in seconds (d30 in our pulseprogram)
%set High_F to the value (in MHz) of the high static field
%set Low_F to the value (in MHz) of the low static field
%set MC_fac to 0 or 1 (if 1, synthetic noise will be added to the data - %for error calculation)
%set do_plots to 0 or 1 (1 is to show plots of the experimental and simulated RD curves
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OUTPUT FILE
%File name: NH_res_based.out
%DWppm is the chemical shift difference (in ppm) between states A and B
%pb: is the fractional population of state B
%kex: is the exchange rate constant (in 1/s) = kab + kba
%R2_HighF is the intrinsic R2 (in 1/s) at the high static field 
%R2_LowF is the intrinsic R2 (in 1/s) at the low static field
%Red.chi2 is the reduced chi squared 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function CS_fit(par,data)
clear

global DATATEMP_40_800;
global DATATEMP_40_600;
global NumFields;
global FULL_CT;
global High_F;
global Low_F;

NumRes=12;
NumFields=15;
FULL_CT=0.06;                                                              % relaxation period in seconds
High_F = 81.108;
Low_F = 60.818;

MC_fac = 0;
do_plots = 1;                                                              % set to 1 to plot data

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load R2 Data.

load ../inputs/HighField.in;
load ../inputs/LowField.in;

NH_40C_800=HighField;
NH_40C_600=LowField;
%% 


Index = NH_40C_800(1,2:NumRes+1)';


  VV=MC_fac*randn(NumFields,NumRes);                                            %create a gaussian distributed random matrix centered at 0 with standard dev 0.1 
  Err_40_800 = NH_40C_800(2:end,NumRes+2:end);                                              
  Err_40_600 = NH_40C_600(2:end,NumRes+2:end);                                              

  Temp_40_800 = NH_40C_800(2:end,2:NumRes+1);                                              
  Temp_40_600 = NH_40C_600(2:end,2:NumRes+1);                                              

  TempF_40_800 = NH_40C_800(2:end,1);                                              
  TempF_40_600 = NH_40C_600(2:end,1);                                              
  
  AV_40_800 = VV.*Err_40_800;
  AV_40_600 = VV.*Err_40_600;

Dis_40_800 = [TempF_40_800 Temp_40_800+AV_40_800 Err_40_800];
Dis_40_600 = [TempF_40_600 Temp_40_600+AV_40_600 Err_40_600];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Initializations
par0=zeros(1, 5);                                                   %create an array of 5 residue based parameters DW800, pb40, k40, R2_800_40, R2_600_40)
parf=zeros(NumRes, 5);                                              %initialize parf where to store residue specific parameters after fitting
DATATEMP_40_800=zeros(NumFields, 3);                                %initialize DATATEMP (data for the current residue)
DATATEMP_40_600=zeros(NumFields, 3);                                %initialize DATATEMP (data for the current residue)

Beff_calc = (20:50:1020)';                                          % make a column vector with elements from 50 to 2100 with intervals of 50 (for plots)
R2s_40_800=zeros(size(Beff_calc,1), 1);                                % create a column of 0 of the same size of Beff_Calc (for plots)
R2s_40_600=zeros(size(Beff_calc,1), 1);                                % create a column of 0 of the same size of Beff_Calc (for plots)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Define starting parameters for fitting
par0(1)=1000;                                                       % guess for DW800  
par0(2)=0.05;                                                       % guess for pb_70
par0(3)=1000;                                                       % guess for Kex_40
par0(4)=30;                                                         % guess for intrinsic R2_800_70
par0(5)=30;                                                         % guess for intrinsic R2_800_60

for k=1:NumRes
        Fitting_residue = Index(k)
        DATATEMP_40_800(:,1)=Dis_40_800(:,1);                           % cpmgfield for current residue at 800 MHz and 40C
        DATATEMP_40_800(:,2)=Dis_40_800(:,k+1);                         % R2 for current residue at 800 MHz and 40C
        DATATEMP_40_800(:,3)=Dis_40_800(:,k+1+NumRes);                  % R2 errors for the current residue in the first column ERRTEMP

        DATATEMP_40_600(:,1)=Dis_40_600(:,1);                           % cpmgfield for current residue at 600 MHz and 40C
        DATATEMP_40_600(:,2)=Dis_40_600(:,k+1);                         % R2 for current residue at 600 MHz and 470C
        DATATEMP_40_600(:,3)=Dis_40_600(:,k+1+NumRes);                  % R2 errors for the current residue in the first column ERRTEMP
        
        initial_chi2=chi2(par0,1)                                 % print out the initial chi2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%fit 
   flag_chi2_or_residuals=2;
   lb=abs(par0)*0;
   ub=abs(par0)*0+inf;

    ub(1)=3000;                                                       %  
   
   tic
   %to generate empirical jacobian
   fprintf('\n\n\nGenerate empirical jacobian\n');
   opts = optimset('Algorithm',   'trust-region-reflective',...
                   'MaxFunEvals',  1e4,...
                   'MaxIter',      10,...
                   'TolFun',       1e-7,...
                   'TolX',         1e-4,...
                   'scaleProblem','Jacobian');
                   %'JacobPattern', jacpat);
   [par,resnorm2,residual2,exitflag2,output2,lambda2,jacobian2]...
                  = lsqnonlin(@chi2, par0, lb, ub, opts, flag_chi2_or_residuals);
%flag_chi2_or_residuals
   
   jacpat=jacobian2; 
   fprintf('\n\n\nFull optimization\n');
   %full optimization
   opts = optimset('Algorithm',   'trust-region-reflective',...
                   'MaxFunEvals',  1e6,...
                   'MaxIter',      20e3,...
                   'TolFun',       1e-11,...
                   'TolX',         1e-9,...
                   'scaleProblem','Jacobian',...
                   'JacobPattern', jacpat);
   [par,resnorm2,residual2,exitflag2,output2,lambda2,jacobian2]...
                  = lsqnonlin(@chi2, par0, lb, ub, opts, flag_chi2_or_residuals);
%              

   timer1=toc;
   fprintf('\nOptimized for %6.2f seconds\n',timer1);           
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%outputs
size(jacpat)
final_chi2=chi2(par,1)       
       chi2f(k)=chi2(par,1)/((NumFields * 2) - 5);
       parf(k,:)=par;                                                % save current parameters in parf

       DW800 = par(1);
       PB_40 = par(2);
       KE_40 = par(3);
       R2_40_800 = par(4);
       R2_40_600 = par(5);

       
       for j=1:size(Beff_calc, 1)                                    
       R2s_40_800(j) = R2_calc_SQ(PB_40, KE_40, R2_40_800, DW800, 1./(4.*Beff_calc(j)));                                    % Simulate R2_900 for the current residue using the final parameters
       R2s_40_600(j) = R2_calc_SQ(PB_40, KE_40, R2_40_600, DW800/High_F*Low_F, 1./(4.*Beff_calc(j)));                                    % Simulate R2_900 for the current residue using the final parameters
       end

  
      
        if(do_plots == 1)
         figure('Position', [400 300 600 350], 'Name', num2str(Index(k)));
         errorbar(DATATEMP_40_800(:,1), DATATEMP_40_800(:,2), DATATEMP_40_800(:,3), 'dr')
         hold on 
         plot(Beff_calc, R2s_40_800, 'r-');
         axis([0 1100 0 1.3*DATATEMP_40_800(1,2)]);
         hold on
         errorbar(DATATEMP_40_600(:,1), DATATEMP_40_600(:,2), DATATEMP_40_600(:,3), 'db')
         hold on 
         plot(Beff_calc, R2s_40_600, 'b-');
         hold off
        end
         
end
       

fout=fopen('NH_res_based.out', 'w');
fprintf(fout, 'Residue specific parameters: index, DWppm, pb, kex, R2_HighF, R2_LowF, Red.chi2.\n');
for i=1:NumRes

       DW800 = parf(i,1);
       pb_40 = parf(i,2);
       KE_40 = parf(i,3);
       R2_40_800 = parf(i,4);
       R2_40_600 = parf(i,5);
       Rchi=chi2f(i);

    fprintf(fout, '%d \t%f \t%f \t%f \t%f \t%f \t%f\n', Index(i), DW800/2/pi/High_F, pb_40, KE_40, R2_40_800, R2_40_600, Rchi);    % print residue specific parametrs in a text file
end
fclose(fout);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate chi2 for a given set of measured and predicted R2s
function F=chi2(par, flag_chi2_or_residuals)
global DATATEMP_40_800;
global DATATEMP_40_600;
global NumFields;
global High_F;
global Low_F;


sum=0;
chi=0;
runningcount=0; %NLF

       DW800 = par(1);
       PB_40 = par(2);
       KE_40 = par(3);
       R2_40_800 = par(4);
       R2_40_600 = par(5);
       
       DW600=DW800/High_F*Low_F;                              % current Delta omega 13C at 800MHz
       R2=zeros(NumFields, 1);
       
% Chi^2 R2(40C) at 800MHz
    field=DATATEMP_40_800(:, 1);                                   % get cpmg field
    data=DATATEMP_40_800(:, 2);                                    % get R2 data for the current residue
    error=DATATEMP_40_800(:, 3);                                   % get errors for the current residue

           for k=1:NumFields                                % for each cpmg field k calulate R2 at 900 and store it in a row R2
            if (data(k)>300)                                            % if you don't have data for residue num at 900MHz put a column of values > 300 in the data file 
            sum=sum;
            else    
            R2(k) = R2_calc_SQ(PB_40, KE_40, R2_40_800, DW800, 1./(4.*field(k)));    
               
            runningcount=runningcount+1;%NLF
            sum=sum+(data(k)-R2(k))^2/error(k)^2;                            % calculate chi2 for all res at 600MHz           
            residuals(runningcount)=(data(k)-R2(k))/error(k);
            end         
           
           end      
    
    
     % Chi^2 R2(40C) at 600MHz
    field=DATATEMP_40_600(:, 1);                                   % get cpmg field
    data=DATATEMP_40_600(:, 2);                                    % get R2 data for the current residue
    error=DATATEMP_40_600(:, 3);                                   % get errors for the current residue

           for k=1:NumFields                                % for each cpmg field k calulate R2 at 900 and store it in a row R2
            if (data(k)>300)                                            % if you don't have data for residue num at 900MHz put a column of values > 300 in the data file 
            sum=sum;
            else    
            R2(k) = R2_calc_SQ(PB_40, KE_40, R2_40_600, DW600, 1./(4.*field(k))); 
           
            runningcount=runningcount+1;%NLF
            sum=sum+(data(k)-R2(k))^2/error(k)^2;                            % calculate chi2 for all res at 600MHz           
            residuals(runningcount)=(data(k)-R2(k))/error(k);
            end         
           
           end  
       
    
    
    
chi=sum;                                                            % reduced chi squared = chi squared / (number of observation - number of fitted parameters -1)
if flag_chi2_or_residuals==1
 F=chi;
elseif flag_chi2_or_residuals==2
 F=residuals;
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate a single value of R2 for a given set of parameters, to be used for plotting
function F=R2_calc_SQ(PB, KE, R2, CW, delta)
global FULL_CT;
            
           PA=1-PB;
           
           PSI = ((PB - PA)*KE)^2 - CW^2 + 4*PA*PB*KE^2;
           NU = 2*CW*(PB-PA)*KE;
           
           D_plus = (1/2)*((PSI + 2*CW^2)/(sqrt(PSI^2 + NU^2)) + 1);
           D_minus = (1/2)*((PSI + 2*CW^2)/(sqrt(PSI^2 + NU^2)) - 1);
           
           eta_plus = (sqrt(2)).*delta.*sqrt( sqrt(PSI^2 + NU^2) + PSI );
           eta_minus = (sqrt(2)).*delta.*sqrt( sqrt(PSI^2 + NU^2) - PSI );
           
           d_plus = CW + 1i * KE;
           d_minus = CW - 1i * KE;
           z_plus = -CW + 1i * KE;
           z_minus = -CW - 1i * KE;
           
           m_d = ( 1i*KE*sqrt(PA*PB)./(d_plus*z_plus)).*(z_plus+2.*CW.*sin(z_plus.*delta)./sin((d_plus + z_plus).*delta));
           m_z = ( -1i*KE*sqrt(PA*PB)./(d_minus*z_minus)).*(d_minus-2.*CW.*sin(d_minus.*delta)./sin((d_minus + z_minus).*delta));
           
           Q = real( 1 - m_d.^2 + m_d.*m_z - m_z.^2 + (1/2).*(m_d + m_z).*sqrt(PB./PA) );
           
           lambda = R2 + (1/2).*( KE - (1./(2.*delta)).*(acosh(D_plus.*cosh(eta_plus) - D_minus.*cos(eta_minus))) );
           R2s = lambda - (1./FULL_CT).*log(Q);
F=R2s;






